"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const client_1 = require("@deep-foundation/hasura/client");
const client_2 = require("../imports/client");
const chai_1 = require("chai");
const util_1 = require("util");
const gql_1 = require("../imports/gql");
const react_1 = require("@testing-library/react");
const client_3 = require("../imports/client");
const react_2 = __importDefault(require("react"));
const client_4 = require("@apollo/client");
require("@testing-library/jest-dom");
const apolloClient = (0, client_1.generateApolloClient)({
    path: `${process.env.DEEPLINKS_HASURA_PATH}/v1/graphql`,
    ssl: !!+process.env.DEEPLINKS_HASURA_SSL,
    secret: process.env.DEEPLINKS_HASURA_SECRET,
    ws: true
});
const deepClient = new client_2.DeepClient({ apolloClient });
describe('client', () => {
    it(`deep.linkId guest and login`, () => __awaiter(void 0, void 0, void 0, function* () {
        chai_1.assert.equal(deepClient.linkId, undefined);
        chai_1.assert.notEqual(deepClient.linkId, 0);
        const guest = yield deepClient.guest();
        const guestDeep = new client_2.DeepClient(Object.assign({ deep: deepClient }, guest));
        chai_1.assert.notEqual(guestDeep.linkId, undefined);
        chai_1.assert.notEqual(guestDeep.linkId, 0);
        const guestId = guestDeep.linkId;
        const adminId = yield deepClient.id('deep', 'admin');
        const admin = yield deepClient.login({ linkId: adminId });
        const deep = new client_2.DeepClient(Object.assign({ deep: deepClient }, admin));
        chai_1.assert.notEqual(deep.linkId, undefined);
        chai_1.assert.notEqual(deep.linkId, 0);
        chai_1.assert.notEqual(deep.linkId, guestId);
    }));
    it(`{ id: 5 }`, () => {
        chai_1.assert.deepEqual(deepClient.serializeWhere({ id: 5 }), { id: { _eq: 5 } });
    });
    it(`{ id: { _eq: 5 } }`, () => {
        chai_1.assert.deepEqual(deepClient.serializeWhere({ id: { _eq: 5 } }), { id: { _eq: 5 } });
    });
    it(`{ value: 5 }`, () => {
        chai_1.assert.deepEqual(deepClient.serializeWhere({ value: 5 }), { number: { value: { _eq: 5 } } });
    });
    it(`{ value: 'a' }`, () => {
        chai_1.assert.deepEqual(deepClient.serializeWhere({ value: 'a' }), { string: { value: { _eq: 'a' } } });
    });
    it(`{ number: 5 }`, () => {
        chai_1.assert.deepEqual(deepClient.serializeWhere({ number: 5 }), { number: { value: { _eq: 5 } } });
    });
    it(`{ string: 'a' }`, () => {
        chai_1.assert.deepEqual(deepClient.serializeWhere({ string: 'a' }), { string: { value: { _eq: 'a' } } });
    });
    it(`{ number: { value: { _eq: 5 } } }`, () => {
        chai_1.assert.deepEqual(deepClient.serializeWhere({ number: { value: { _eq: 5 } } }), { number: { value: { _eq: 5 } } });
    });
    it(`{ string: { value: { _eq: 'a' } } }`, () => {
        chai_1.assert.deepEqual(deepClient.serializeWhere({ string: { value: { _eq: 'a' } } }), { string: { value: { _eq: 'a' } } });
    });
    it(`{ object: { value: { _contains: { a: 'b' } } } }`, () => {
        chai_1.assert.deepEqual(deepClient.serializeWhere({ object: { value: { _contains: { a: 'b' } } } }), { object: { value: { _contains: { a: 'b' } } } });
    });
    it(`{ from: { value: 5 } }`, () => {
        chai_1.assert.deepEqual(deepClient.serializeWhere({ from: { value: 5 } }), { from: { number: { value: { _eq: 5 } } } });
    });
    it(`{ out: { type_id: Contain, value: item, from: where } }`, () => __awaiter(void 0, void 0, void 0, function* () {
        chai_1.assert.deepEqual(deepClient.serializeWhere({
            out: {
                type_id: yield deepClient.id('@deep-foundation/core', 'Contain'),
                value: 'b',
                from: {
                    type_id: yield deepClient.id('@deep-foundation/core', 'Package'),
                    value: 'a',
                },
            },
        }), {
            out: {
                type_id: { _eq: yield deepClient.id('@deep-foundation/core', 'Contain') },
                string: { value: { _eq: 'b' } },
                from: {
                    type_id: { _eq: yield deepClient.id('@deep-foundation/core', 'Package') },
                    string: { value: { _eq: 'a' } },
                },
            }
        });
    }));
    it(`{ value: 5, link: { type_id: 7 } }`, () => {
        chai_1.assert.deepEqual(deepClient.serializeWhere({ value: 5, link: { type_id: 7 } }, 'value'), {
            value: { _eq: 5 },
            link: {
                type_id: { _eq: 7 }
            },
        });
    });
    it(`{ type: ['@deep-foundation/core', 'Value'] }`, () => {
        chai_1.assert.deepEqual(deepClient.serializeWhere({
            type: ["@deep-foundation/core", "Value"],
        }), {
            type: {
                in: {
                    from: {
                        string: { value: { _eq: "@deep-foundation/core" } },
                        type_id: { _eq: 2 },
                    },
                    string: { value: { _eq: "Value" } },
                    type_id: { _eq: 3 },
                },
            },
        });
    });
    it(`{ type: ['@deep-foundation/core', 'Value'] }`, () => {
        chai_1.assert.deepEqual(deepClient.serializeWhere({
            _or: [{
                    type: ["@deep-foundation/core", "Value"],
                }, {
                    type: ["@deep-foundation/core", "User"],
                }]
        }), {
            _or: [{
                    type: {
                        in: {
                            from: {
                                string: { value: { _eq: "@deep-foundation/core" } },
                                type_id: { _eq: 2 },
                            },
                            string: { value: { _eq: "Value" } },
                            type_id: { _eq: 3 },
                        },
                    },
                }, {
                    type: {
                        in: {
                            from: {
                                string: { value: { _eq: "@deep-foundation/core" } },
                                type_id: { _eq: 2 },
                            },
                            string: { value: { _eq: "User" } },
                            type_id: { _eq: 3 },
                        },
                    },
                }]
        });
    });
    it(`id(packageName,contain)`, () => __awaiter(void 0, void 0, void 0, function* () {
        const id = yield deepClient.id('@deep-foundation/core', 'Value');
        chai_1.assert.equal(id, 4);
    }));
    it(`{ type_id: { _type_of: 25 } }`, () => {
        chai_1.assert.deepEqual(deepClient.serializeWhere({ type_id: { _type_of: 25 } }), { type: { _by_item: { path_item_id: { _eq: 25 }, group_id: { _eq: 0 } } } });
    });
    it(`{ from_id: { _type_of: 25 } }`, () => {
        chai_1.assert.deepEqual(deepClient.serializeWhere({ from_id: { _type_of: 25 } }), { from: { _by_item: { path_item_id: { _eq: 25 }, group_id: { _eq: 0 } } } });
    });
    it(`{ to_id: { _type_of: 25 } }`, () => {
        chai_1.assert.deepEqual(deepClient.serializeWhere({ to_id: { _type_of: 25 } }), { to: { _by_item: { path_item_id: { _eq: 25 }, group_id: { _eq: 0 } } } });
    });
    it(`{ id: { _id: ['@deep-foundation/core', 'Package'] } }`, () => __awaiter(void 0, void 0, void 0, function* () {
        var _a, _b, _c;
        chai_1.assert.deepEqual((_c = (_b = (_a = (yield deepClient.select({ id: { _id: ['@deep-foundation/core', 'Package'] } }))) === null || _a === void 0 ? void 0 : _a.data) === null || _b === void 0 ? void 0 : _b[0]) === null || _c === void 0 ? void 0 : _c.id, 2);
    }));
    it(`{ type_id: { _id: ['@deep-foundation/core', 'Package'] } }`, () => __awaiter(void 0, void 0, void 0, function* () {
        var _d, _e;
        const packageId = yield deepClient.id('@deep-foundation/core');
        chai_1.assert.isTrue(!!((_e = (_d = (yield deepClient.select({ type_id: { _id: ['@deep-foundation/core', 'Package'] } }))) === null || _d === void 0 ? void 0 : _d.data) === null || _e === void 0 ? void 0 : _e.find(p => p.id === packageId)));
    }));
    it(`{ up: { parent_id: { _id: ['@deep-foundation/core', 'Package'] } } }`, () => __awaiter(void 0, void 0, void 0, function* () {
        var _f, _g;
        const packageId = yield deepClient.id('@deep-foundation/core');
        chai_1.assert.isTrue(!!((_g = (_f = (yield deepClient.select({ up: { parent_id: { _id: ['@deep-foundation/core', 'Package'] } } }))) === null || _f === void 0 ? void 0 : _f.data) === null || _g === void 0 ? void 0 : _g.find(p => p.id === packageId)));
    }));
    it(`idLocal get from minilinks`, () => __awaiter(void 0, void 0, void 0, function* () {
        const typeTypeLinkId = yield deepClient.id("@deep-foundation/core", "Type");
        const containTypeLinkId = yield deepClient.id("@deep-foundation/core", "Contain");
        const packageTypeLinkId = yield deepClient.id("@deep-foundation/core", "Package");
        const packageName = "idLocal get from minilinks package";
        const { data: [packageLink] } = yield deepClient.insert({
            type_id: packageTypeLinkId,
            string: {
                data: {
                    value: packageName
                }
            }
        }, { returning: deepClient.selectReturning });
        const { data: [newTypeTypeLink] } = yield deepClient.insert({
            type_id: typeTypeLinkId,
        }, { returning: deepClient.selectReturning });
        const { data: [containLink] } = yield deepClient.insert({
            type_id: containTypeLinkId,
            from_id: packageLink.id,
            to_id: newTypeTypeLink.id,
            string: {
                data: {
                    value: "Type"
                }
            }
        }, { returning: deepClient.selectReturning });
        deepClient.minilinks.apply([packageLink, containLink, newTypeTypeLink]);
        try {
            const newTypeTypeLinkId = deepClient.idLocal(packageName, "Type");
            chai_1.assert.notEqual(newTypeTypeLinkId, 0);
            chai_1.assert.equal(newTypeTypeLinkId, newTypeTypeLink.id);
        }
        finally {
            yield deepClient.delete([packageLink.id, newTypeTypeLink.id, containLink.id]);
        }
    }));
    it(`insert link with reserved id and string value`, () => __awaiter(void 0, void 0, void 0, function* () {
        const typeTypeLinkId = yield deepClient.id("@deep-foundation/core", "Type");
        const reservedIds = yield deepClient.reserve(1);
        try {
            yield deepClient.insert({
                id: reservedIds[0],
                type_id: typeTypeLinkId,
                string: {
                    data: {
                        value: 'stringValue'
                    }
                }
            });
        }
        finally {
            yield deepClient.delete(reservedIds[0]);
        }
    }));
    describe(`serial`, () => {
        describe('insert', () => {
            it('one insert', () => __awaiter(void 0, void 0, void 0, function* () {
                let linkIdsToDelete = [];
                const typeTypeLinkId = yield deepClient.id("@deep-foundation/core", "Type");
                const operation = (0, gql_1.createSerialOperation)({
                    table: 'links',
                    type: 'insert',
                    objects: {
                        type_id: typeTypeLinkId
                    }
                });
                try {
                    const result = yield deepClient.serial({
                        operations: [
                            operation,
                        ]
                    });
                    chai_1.assert.equal(result.error, undefined);
                    chai_1.assert.notEqual(result.data, undefined);
                    linkIdsToDelete = [...linkIdsToDelete, ...result.data.map(link => link.id)];
                    chai_1.assert.strictEqual(result.data.length, 1);
                }
                finally {
                    yield deepClient.delete(linkIdsToDelete);
                }
            }));
            it('multiple inserts in one operation', () => __awaiter(void 0, void 0, void 0, function* () {
                let linkIdsToDelete = [];
                const typeTypeLinkId = yield deepClient.id("@deep-foundation/core", "Type");
                const operation = (0, gql_1.createSerialOperation)({
                    table: 'links',
                    type: 'insert',
                    objects: {
                        type_id: typeTypeLinkId
                    }
                });
                try {
                    const result = yield deepClient.serial({
                        operations: [
                            operation,
                            operation
                        ]
                    });
                    chai_1.assert.equal(result.error, undefined);
                    chai_1.assert.notEqual(result.data, undefined);
                    linkIdsToDelete = [...linkIdsToDelete, ...result.data.map(link => link.id)];
                    chai_1.assert.strictEqual(result.data.length, 2);
                }
                finally {
                    yield deepClient.delete(linkIdsToDelete);
                }
            }));
            it('multiple inserts ine multiple operations', () => __awaiter(void 0, void 0, void 0, function* () {
                let linkIdsToDelete = [];
                const typeTypeLinkId = yield deepClient.id("@deep-foundation/core", "Type");
                const operation = (0, gql_1.createSerialOperation)({
                    table: 'links',
                    type: 'insert',
                    objects: {
                        type_id: typeTypeLinkId
                    }
                });
                try {
                    const result = yield deepClient.serial({
                        operations: [
                            operation,
                            operation
                        ]
                    });
                    console.log((0, util_1.inspect)(result));
                    chai_1.assert.equal(result.error, undefined);
                    chai_1.assert.notEqual(result.data, undefined);
                    linkIdsToDelete = [...linkIdsToDelete, ...result.data.map(link => link.id)];
                    chai_1.assert.strictEqual(result.data.length, 2);
                }
                finally {
                    yield deepClient.delete(linkIdsToDelete);
                }
            }));
        });
        describe('update', () => {
            it('one update', () => __awaiter(void 0, void 0, void 0, function* () {
                let linkIdsToDelete = [];
                const expectedValue = 'newStringValue';
                const typeTypeLinkId = yield deepClient.id("@deep-foundation/core", "Type");
                try {
                    const insertResult = yield deepClient.insert({
                        type_id: typeTypeLinkId,
                        string: {
                            data: {
                                value: "stringValue"
                            }
                        }
                    });
                    chai_1.assert.equal(insertResult.error, undefined);
                    chai_1.assert.notEqual(insertResult.data, undefined);
                    chai_1.assert.strictEqual(insertResult.data.length, 1);
                    const newLinkId = insertResult.data[0].id;
                    linkIdsToDelete.push(newLinkId);
                    const operation = (0, gql_1.createSerialOperation)({
                        table: 'strings',
                        type: 'update',
                        exp: {
                            link_id: newLinkId,
                        },
                        value: {
                            value: expectedValue
                        }
                    });
                    const updateResult = yield deepClient.serial({
                        operations: [
                            operation
                        ]
                    });
                    chai_1.assert.equal(updateResult.error, undefined);
                    chai_1.assert.notEqual(updateResult.data, undefined);
                    chai_1.assert.strictEqual(updateResult.data.length, 1);
                    const { data: [newLink] } = yield deepClient.select({
                        id: newLinkId
                    });
                    chai_1.assert.strictEqual(newLink.value.value, expectedValue);
                }
                finally {
                    yield deepClient.delete(linkIdsToDelete);
                }
            }));
            it('multiple updates in one operation', () => __awaiter(void 0, void 0, void 0, function* () {
                let linkIdsToDelete = [];
                const expectedValue = 'newStringValue';
                const typeTypeLinkId = yield deepClient.id("@deep-foundation/core", "Type");
                try {
                    const insertResult = yield deepClient.insert({
                        type_id: typeTypeLinkId,
                        string: {
                            data: {
                                value: "stringValue"
                            }
                        }
                    });
                    chai_1.assert.equal(insertResult.error, undefined);
                    chai_1.assert.notEqual(insertResult.data, undefined);
                    chai_1.assert.strictEqual(insertResult.data.length, 1);
                    const newLinkId = insertResult.data[0].id;
                    linkIdsToDelete.push(newLinkId);
                    const operation = (0, gql_1.createSerialOperation)({
                        table: 'strings',
                        type: 'update',
                        exp: {
                            link_id: newLinkId,
                        },
                        value: {
                            value: expectedValue
                        }
                    });
                    const updateResult = yield deepClient.serial({
                        operations: [
                            operation,
                            operation
                        ]
                    });
                    chai_1.assert.equal(updateResult.error, undefined);
                    chai_1.assert.notEqual(updateResult.data, undefined);
                    chai_1.assert.strictEqual(updateResult.data.length, 2);
                    const { data: [newLink] } = yield deepClient.select({
                        id: newLinkId
                    });
                    chai_1.assert.strictEqual(newLink.value.value, expectedValue);
                }
                finally {
                    yield deepClient.delete(linkIdsToDelete);
                }
            }));
            it('multiple updates in multiple operations', () => __awaiter(void 0, void 0, void 0, function* () {
                let linkIdsToDelete = [];
                const expectedValue = 'newStringValue';
                const typeTypeLinkId = yield deepClient.id("@deep-foundation/core", "Type");
                try {
                    const insertResult = yield deepClient.insert({
                        type_id: typeTypeLinkId,
                        string: {
                            data: {
                                value: "stringValue"
                            }
                        }
                    });
                    chai_1.assert.equal(insertResult.error, undefined);
                    chai_1.assert.notEqual(insertResult.data, undefined);
                    chai_1.assert.strictEqual(insertResult.data.length, 1);
                    const newLinkId = insertResult.data[0].id;
                    linkIdsToDelete.push(newLinkId);
                    const operation = (0, gql_1.createSerialOperation)({
                        table: 'strings',
                        type: 'update',
                        exp: {
                            link_id: newLinkId,
                        },
                        value: {
                            value: expectedValue
                        }
                    });
                    const updateResult = yield deepClient.serial({
                        operations: [
                            operation,
                            operation
                        ]
                    });
                    chai_1.assert.equal(updateResult.error, undefined);
                    chai_1.assert.notEqual(updateResult.data, undefined);
                    chai_1.assert.strictEqual(updateResult.data.length, 2);
                    const { data: [newLink] } = yield deepClient.select({
                        id: newLinkId
                    });
                    chai_1.assert.strictEqual(newLink.value.value, expectedValue);
                }
                finally {
                    yield deepClient.delete(linkIdsToDelete);
                }
            }));
        });
        describe('delete', () => {
            it('one delete', () => __awaiter(void 0, void 0, void 0, function* () {
                let linkIdsToDelete = [];
                const typeTypeLinkId = yield deepClient.id("@deep-foundation/core", "Type");
                try {
                    const { data: [{ id: newLinkId }] } = yield deepClient.insert({
                        type_id: typeTypeLinkId,
                        string: {
                            data: {
                                value: "stringValue"
                            }
                        }
                    });
                    linkIdsToDelete.push(newLinkId);
                    const operation = (0, gql_1.createSerialOperation)({
                        table: 'links',
                        type: 'delete',
                        exp: {
                            id: newLinkId
                        }
                    });
                    const result = yield deepClient.serial({
                        operations: [
                            operation,
                        ]
                    });
                    chai_1.assert.equal(result.error, undefined);
                    chai_1.assert.notEqual(result.data, undefined);
                    chai_1.assert.strictEqual(result.data.length, 1);
                    const { data: [newLink] } = yield deepClient.select(newLinkId);
                    chai_1.assert.equal(newLink, undefined);
                }
                finally {
                    yield deepClient.delete(linkIdsToDelete);
                }
            }));
            it('multiple deletes in one operation', () => __awaiter(void 0, void 0, void 0, function* () {
                let linkIdsToDelete = [];
                try {
                    const reservedLinkIds = yield deepClient.reserve(2);
                    linkIdsToDelete = [...linkIdsToDelete, ...reservedLinkIds];
                    const result = yield deepClient.serial({
                        operations: [
                            {
                                table: 'links',
                                type: 'delete',
                                exp: {
                                    id: reservedLinkIds[0]
                                }
                            },
                            {
                                table: 'links',
                                type: 'delete',
                                exp: {
                                    id: reservedLinkIds[1]
                                }
                            }
                        ]
                    });
                    chai_1.assert.equal(result.error, undefined);
                    chai_1.assert.notEqual(result.data, undefined);
                    chai_1.assert.strictEqual(result.data.length, 2);
                    const { data } = yield deepClient.select(reservedLinkIds);
                    chai_1.assert.equal(data.length, 0);
                }
                finally {
                    yield deepClient.delete(linkIdsToDelete);
                }
            }));
            it('multiple deletes in multiple operations', () => __awaiter(void 0, void 0, void 0, function* () {
                let linkIdsToDelete = [];
                try {
                    const reservedLinkIds = yield deepClient.reserve(2);
                    linkIdsToDelete = [...linkIdsToDelete, ...reservedLinkIds];
                    const result = yield deepClient.serial({
                        operations: [
                            {
                                table: 'links',
                                type: 'delete',
                                exp: {
                                    id: reservedLinkIds[0]
                                }
                            },
                            {
                                table: 'links',
                                type: 'delete',
                                exp: {
                                    id: reservedLinkIds[1]
                                }
                            }
                        ]
                    });
                    chai_1.assert.equal(result.error, undefined);
                    chai_1.assert.notEqual(result.data, undefined);
                    chai_1.assert.strictEqual(result.data.length, 2);
                    const { data } = yield deepClient.select(reservedLinkIds);
                    chai_1.assert.equal(data.length, 0);
                }
                finally {
                    yield deepClient.delete(linkIdsToDelete);
                }
            }));
        });
    });
    it('select string table', () => __awaiter(void 0, void 0, void 0, function* () {
        const typeTypeLinkId = yield deepClient.id("@deep-foundation/core", "Type");
        const { data: [newLink] } = yield deepClient.insert({
            type_id: typeTypeLinkId,
            string: {
                data: {
                    value: "stringValue"
                }
            }
        });
        yield deepClient.select({
            link_id: { _eq: newLink.id }
        }, {
            table: 'strings'
        });
    }));
    describe(`useDeepSubscription`, () => {
        it(`type`, () => __awaiter(void 0, void 0, void 0, function* () {
            yield setup();
            expect(react_1.screen.getByText('Loading...')).toBeInTheDocument();
            yield (0, react_1.waitFor)(() => {
                expect(react_1.screen.queryByText('Loading...')).not.toBeInTheDocument();
            });
            expect(react_1.screen.queryByText(/^Error:/)).not.toBeInTheDocument();
            yield (0, react_1.waitFor)(() => {
                const dataLengthText = react_1.screen.getByText(/items loaded$/);
                expect(dataLengthText).toBeInTheDocument();
                const dataLength = parseInt(dataLengthText.textContent);
                expect(dataLength).toBeGreaterThan(0);
            }, { timeout: 10000 });
            function setup() {
                return __awaiter(this, void 0, void 0, function* () {
                    const typeTypeLinkId = yield deepClient.id("@deep-foundation/core", "Type");
                    function TestHookComponent() {
                        const { loading, data, error } = (0, client_2.useDeepSubscription)({
                            type_id: typeTypeLinkId,
                        });
                        if (loading) {
                            return react_2.default.createElement("div", null, "Loading...");
                        }
                        if (error) {
                            return react_2.default.createElement("div", null,
                                "Error: ",
                                error.message);
                        }
                        return react_2.default.createElement("div", null,
                            data.length,
                            " items loaded");
                    }
                    (0, react_1.render)(react_2.default.createElement(client_4.ApolloProvider, { client: deepClient.apolloClient },
                        react_2.default.createElement(client_3.DeepProvider, null,
                            react_2.default.createElement(TestHookComponent, null))));
                });
            }
        }));
    });
});
//# sourceMappingURL=client.js.map